@@ -534,7 +534,9 @@ emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
         contr = attr(model.matrix(object), "contrasts")
     X = model.matrix(trms, m, contrasts.arg = contr)
     bhat = rstanarm::fixef(object)
-    V = vcov(object)
+    nms = intersect(colnames(X), names(bhat))
+    bhat = bhat[nms]
+    V = vcov(object)[nms, nms]
     misc = list()
     if (!is.null(object$family)) {
         if (is.character(object$family)) # work around bug for stan_polr
@@ -549,7 +551,7 @@ emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
             mode = match.arg(mode, 
                              c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"))
         
-        xint = match("(Intercept)", colnames(X), nomatch = 0L)
+        xint = match("(Intercept)", nms, nomatch = 0L)
         if (xint > 0L) 
             X = X[, -xint, drop = FALSE]
         k = length(object$zeta)
@@ -575,14 +577,14 @@ emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
             }
         }
         
-        misc$respName = as.character(terms(object))[2]
+        misc$respName = as.character.default(terms(object))[2]
     }
-    samp = as.matrix(object$stanfit)[, names(bhat), drop = FALSE]
+    samp = as.matrix(object$stanfit)[, nms, drop = FALSE]
     attr(samp, "n.chains") = object$stanfit@sim$chains
     # estimability...
     nbasis = estimability::all.estble
     all.nms = colnames(X)
-    if (length(names(bhat)) < (n <- length(all.nms))) {
+    if (length(nms) < length(all.nms)) {
         coef = NA * X[1, ]
         coef[names(bhat)] = bhat
         bhat = coef
